"use strict";
/**
 * @file logger
 * @author Cuttle Cong
 * @date 2018/3/27
 * @description
 */
Object.defineProperty(exports, "__esModule", { value: true });
const logSymbols = require('log-symbols');
function concat(head, arr, label) {
    const out = [];
    if (typeof head === 'string' || head instanceof String) {
        out.push(label + ' ' + head);
    }
    else {
        out.push(label);
        out.push(head);
    }
    return out.concat(arr);
}
class DefaultLogger {
    constructor() {
        this.silent = false;
        this._log = (head, ...arr) => {
            console.log.apply(console, concat(head, arr, logSymbols.info));
        };
        this._success = (head, ...arr) => {
            console.log.apply(console, concat(head, arr, logSymbols.success));
        };
        this._warn = (head, ...arr) => {
            console.warn.apply(console, concat(head, arr, logSymbols.warning));
        };
        this._error = (head, ...arr) => {
            console.error.apply(console, concat(head, arr, logSymbols.error));
        };
        this.log = (head, ...arr) => {
            !this.silent && this._log(head, ...arr);
        };
        this.success = (head, ...arr) => {
            !this.silent && this._success(head, ...arr);
        };
        this.warn = (head, ...arr) => {
            !this.silent && this._warn(head, ...arr);
        };
        this.deprecated = (...arr) => {
            this.warn('[Deprecated]', ...arr);
        };
        this.error = (head, ...arr) => {
            this._error(head, ...arr);
        };
    }
}
exports.default = DefaultLogger;
