"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const normalizeSource_1 = require("./normalizeSource");
const loadConfig_1 = require("../lib/loadConfig");
const extendsMerge_1 = require("./extendsMerge");
const extendsConfig_1 = require("./extendsConfig");
const _ = require("lodash");
const nps = require("path");
const constant_1 = require("./constant");
const tildify = require('tildify');
const debug = require('debug')('edam:normalizeConfig');
/**
 *
 * @param {EdamConfig} looseConfig
 * @param {Options} options
 * @return {Promise<EdamConfig>}
 */
function normalizeConfig(looseConfig, options = { cwd: process.cwd() }) {
    return __awaiter(this, void 0, void 0, function* () {
        debug('input: loose Config %O', looseConfig);
        debug('input: options %o', options);
        looseConfig = Object.assign({
            userc: true,
            yes: false,
            silent: false,
            extends: [],
            alias: {},
            cacheDir: true,
            updateNotify: true
        }, looseConfig);
        // if (!looseConfig.plugins || !looseConfig.plugins.length) {
        //   delete looseConfig.plugins
        // }
        const coreSpecial = {
            userc: looseConfig.userc,
            yes: looseConfig.yes,
            silent: looseConfig.silent,
            updateNotify: looseConfig.updateNotify,
            name: looseConfig.name
        };
        // merge extends Configuration
        let { config: mergedConfig, track } = yield extendsConfig_1.default(looseConfig, Object.assign({}, options, { track: true }));
        // mergedConfig = _.cloneDeep(mergedConfig)
        if (coreSpecial.userc) {
            const obj = yield loadConfig_1.load(options.cwd);
            debug('rc config: %o', obj);
            if (obj) {
                const { config: rcConfig, filepath } = obj;
                const mergedRcConfig = yield extendsConfig_1.innerExtendsConfig(rcConfig, { cwd: nps.dirname(filepath) }, track);
                debug('rc merged config: %O', mergedRcConfig);
                debug('mergedConfig config before: %O', mergedConfig);
                mergedConfig = extendsMerge_1.default({}, mergedRcConfig, mergedConfig);
            }
        }
        debug('merged config after: %O', mergedConfig);
        if (!mergedConfig.plugins) {
            mergedConfig.plugins = [];
        }
        // normalize source
        _.each(mergedConfig.alias, (val, key) => {
            mergedConfig.alias[key] = normalizeSource_1.default(mergedConfig.alias[key], options);
        });
        // Given source is alias
        if (mergedConfig.source) {
            let sourceUrl = typeof mergedConfig.source === 'string'
                ? mergedConfig.source
                : mergedConfig.source.url;
            const data = Object.assign({}, (mergedConfig.source || {}));
            delete data.url;
            delete data.type;
            let source = mergedConfig.source;
            if (_.isString(sourceUrl)) {
                // mergedConfig.source append with querystring
                let tmpSource = sourceUrl;
                if (tmpSource in mergedConfig.alias) {
                    source = Object.assign({}, mergedConfig.alias[tmpSource], data, { config: Object.assign({}, mergedConfig.alias[tmpSource].config, data.config) });
                }
            }
            if (source.type !== 'npm') {
                delete source.version;
            }
            if (source.type !== 'git') {
                delete source.checkout;
            }
            mergedConfig.source = source;
        }
        // normalize cacheDir
        if (_.isString(mergedConfig.cacheDir)) {
            mergedConfig.cacheDir = nps.resolve(options.cwd, (mergedConfig.cacheDir));
        }
        else if (mergedConfig.cacheDir) {
            mergedConfig.cacheDir = constant_1.default.DEFAULT_CACHE_DIR;
        }
        if (typeof mergedConfig.storePrompts === 'undefined') {
            mergedConfig.storePrompts = true;
        }
        let sourceConfig = mergedConfig.source
            ? mergedConfig.source.config || {}
            : {};
        debug('sourceConfig: %O', sourceConfig);
        const normalized = Object.assign({}, mergedConfig, sourceConfig, { pull: Object.assign({ npmClient: 'npm' }, mergedConfig.pull, sourceConfig.pull) }, coreSpecial);
        debug('normalized Config: %O', normalized);
        return { config: normalized, track };
    });
}
exports.default = normalizeConfig;
