"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * @file normalizeSource
 * @author Cuttle Cong
 * @date 2018/3/23
 * @description
 */
const parseGitUrl_1 = require("../lib/parseGitUrl");
const isGitUrl_1 = require("../lib/isGitUrl");
const fileSystem_1 = require("../lib/fileSystem");
const nps = require("path");
const _ = require("lodash");
const resolve_1 = require("../lib/resolve");
const extendsConfig_1 = require("./extendsConfig");
const parseQuery_1 = require("../lib/parseQuery");
const debug = require('debug')('edam:normalizeSource');
const untildify = require('untildify');
function tran(ref, name, fn) {
    if (ref[name] != null) {
        ref[name] = fn(ref[name]);
    }
}
function parsePureGitUrl(source) {
    const parsed = parseGitUrl_1.default(source);
    return parsed.url.replace(/(\.git)?$/, '.git');
}
function parseGit(source) {
    return {
        type: 'git',
        url: parsePureGitUrl(source),
        checkout: parseGitUrl_1.default(source).checkout
    };
}
function normalizeSourceObject(source, options) {
    const { cwd = process.cwd() } = options || {};
    if (source.type === 'file') {
        source.url = nps.resolve(cwd, source.url);
    }
    if (source.type === 'git') {
        source.url = parsePureGitUrl(source.url);
        if (!source.checkout) {
            source.checkout = 'master';
        }
    }
    if (source.config) {
        tran(source.config, 'output', str => nps.resolve(cwd, str));
        tran(source.config, 'cacheDir', cacheDir => {
            if (_.isString(cacheDir)) {
                return nps.resolve(cwd, cacheDir);
            }
            return cacheDir;
        });
        tran(source.config, 'plugins', plugins => extendsConfig_1.normalizePlugins(plugins, options));
    }
    return source;
}
exports.normalizeSourceObject = normalizeSourceObject;
function parseNpm(source) {
    let version = '';
    if (/^(.+)@([^@]*)$/.test(source)) {
        let matchedSource = RegExp.$1;
        version = RegExp.$2;
        source = matchedSource;
    }
    return {
        type: 'npm',
        url: source,
        version
    };
}
function normalizeSource(source, options) {
    // const { cwd = process.cwd() } = options || {}
    if (_.isObject(source)) {
        return normalizeSourceObject(source, options);
    }
    if (!_.isString(source)) {
        return source;
    }
    let result;
    let filename;
    debug('input source: %s', source);
    source = source.trim();
    let prefix = '';
    if (/^(.+?):/.test(source)) {
        prefix = RegExp.$1;
    }
    if (['npm'].includes(prefix)) {
        source = source.substring(prefix.length + 1);
        return parseNpm(source);
    }
    if (((filename = resolve_1.default(untildify(source), Object.assign({}, options, { safe: true }))),
        fileSystem_1.default.isFile(filename))) {
        result = {
            type: 'file',
            url: filename
        };
    }
    else if (isGitUrl_1.default(source)) {
        result = parseGit(source);
    }
    else {
        result = parseNpm(source);
    }
    let i = result.url.lastIndexOf('?');
    if (i >= 0) {
        result = Object.assign({}, parseQuery_1.default(result.url.slice(i)), result, { url: result.url.slice(0, i) });
    }
    debug('source parsed: %o', result);
    return result;
}
exports.default = normalizeSource;
