"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const getExtendsMerge_1 = require("../../lib/getExtendsMerge");
const _ = require("lodash");
const toArray_1 = require("../../lib/toArray");
const path_1 = require("path");
const npmInstall = require('../../lib/yarnInstall');
const debug = require('debug')('edam:adaptor');
function filter() {
    return __awaiter(this, void 0, void 0, function* () {
        const edam = this;
        edam.prependOnceListener('normalize:templateConfig:after', () => {
            const templateConfig = edam.templateConfig;
            let { loaders, mappers, hooks } = templateConfig;
            const compiler = edam.compiler;
            const variables = edam.compiler.variables;
            debug('compiler preset loaders %O', compiler.loaders);
            debug('compiler preset mappers %O', compiler.mappers);
            debug('input mappers %O', mappers);
            getExtendsMerge_1.default({ concatKeys: ['mappers'] })(compiler, {
                loaders,
                mappers: mappers || []
            });
            debug('compiler merged loaders %O', compiler.loaders);
            debug('compiler merged mappers %O', compiler.mappers);
            if (templateConfig.usefulHook) {
                const { gitInit, installDependencies, installDevDependencies } = templateConfig.usefulHook;
                if (gitInit) {
                    compiler.addHook('post', 'cd $HOOK_PARAMS_0 && git init', {
                        type: 'once',
                        silent: edam.config.silent
                    });
                }
                if (installDependencies || installDevDependencies) {
                    compiler.addHook('post', function (output) {
                        return __awaiter(this, void 0, void 0, function* () {
                            yield edam.emit('install:packages:before');
                            yield npmInstall({
                                production: installDependencies && !installDevDependencies,
                                dev: !installDependencies && installDevDependencies,
                                stdio: 'pipe',
                                cwd: path_1.resolve(output)
                            });
                            yield edam.emit('install:packages:after');
                        });
                    }, {
                        type: 'once',
                        silent: edam.config.silent
                    });
                }
            }
            _.each(hooks, (hook, key) => {
                toArray_1.default(hook).forEach(hook => {
                    compiler.addHook(key, hook, {
                        type: 'on',
                        silent: edam.config.silent
                    });
                });
            });
            variables.assign(templateConfig.variables);
        });
    });
}
exports.default = filter;
