"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * @file filter
 * @author Cuttle Cong
 * @date 2018/3/24
 * @description
 */
const fileSystem_1 = require("../../lib/fileSystem");
const nps = require("path");
const _ = require("lodash");
const mm = require("micromatch");
const EdamError_1 = require("../EdamError");
const tildify = require('tildify');
const debug = require('debug')('edam:filter');
function filter() {
    return __awaiter(this, void 0, void 0, function* () {
        const edam = this;
        edam.prependOnceListener('normalize:templateConfig:after', (templateConfig) => __awaiter(this, void 0, void 0, function* () {
            // const templateConfig = edam.templateConfig
            if (!templateConfig.root) {
                throw new EdamError_1.default('templateConfig.root is required on pre treat: filter');
            }
            const compiler = edam.compiler;
            let { root, ignore } = templateConfig;
            if (!fileSystem_1.default.isDirectory(root)) {
                throw new EdamError_1.default(`templateConfig.root ${tildify(root)} is illegal, maybe it is not existed or is file path.`);
            }
            if (_.isFunction(ignore)) {
                ignore = yield ignore(compiler.variables);
            }
            let files = yield fileSystem_1.default.readdirDeep(root, {
                filter: [`!${edam.templateConfigPath}`]
            });
            debug('assets files: %O', files);
            files = mm(files.map(f => nps.relative(root, f)), ['*'], {
                ignore: ignore,
                dot: true,
                matchBase: true
            }).map(f => nps.join(root, f));
            // deep read dir and filter
            debug('assets filter files: %O', files);
            yield Promise.all(files.map(function (file) {
                return __awaiter(this, void 0, void 0, function* () {
                    const name = nps.relative(root, file);
                    compiler.assets[name] = {
                        value: yield fileSystem_1.default.readFile(file, { encoding: 'utf8' })
                    };
                });
            }));
        }));
    });
}
exports.default = filter;
