"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const sourceFilenamify_1 = require("../sourceFilenamify");
const path_1 = require("path");
const mkdirp = require("mkdirp");
const fileSystem_1 = require("../../lib/fileSystem");
const chalk_1 = require("chalk");
const EdamError_1 = require("../EdamError");
const tildify = require('tildify');
const updateNotifier = require('update-notifier');
const c = require('chalk');
const debug = require('debug')('edam:pull:npm');
const semver = require('semver');
const yarnInstall = require('../../lib/yarnInstall');
module.exports = function npmPull(source, destDir, options) {
    return __awaiter(this, void 0, void 0, function* () {
        const { cacheDir, pull: { npmClient } } = options;
        const log = (this && this.logger && this.logger.log) || console.log;
        const updateNotify = this && this.config && this.config.updateNotify;
        let respectNpm5 = npmClient === 'npm';
        const name = source.version ? `${source.url}@${source.version}` : source.url;
        let dest;
        let modulePath;
        let oldPkgPath;
        const install = () => __awaiter(this, void 0, void 0, function* () {
            try {
                yield yarnInstall([name], {
                    respectNpm5,
                    stdio: 'pipe',
                    cwd: dest
                    // showCommand: true
                });
            }
            catch (err) {
                throw new EdamError_1.default(`Install package "${name}" failed. \n` +
                    (err && err.stack));
            }
            if (updateNotify) {
                const notifier = updateNotifier({
                    pkg: JSON.parse(yield fileSystem_1.default.readFile(path_1.join(modulePath, 'package.json'), {
                        encoding: 'utf8'
                    }))
                });
                const output = this && this.config ? tildify(this.config.output) : './output';
                const upt = notifier.update;
                if (upt) {
                    notifier.notify({
                        message: 'Update available ' +
                            chalk_1.default.dim(upt.current) +
                            chalk_1.default.reset(' → ') +
                            chalk_1.default.green(upt.latest) +
                            ' \nRun ' +
                            chalk_1.default.cyan(`edam ${source.url}@latest ${output}`) +
                            ' to update'
                    });
                }
            }
            const installedVersion = JSON.parse(yield fileSystem_1.default.readFile(oldPkgPath, { encoding: 'utf8' })).version;
            log('Installed %s from NPM, and the actual version is %s.', c.magenta.bold(name), c.green(installedVersion));
        });
        dest = path_1.join(destDir, sourceFilenamify_1.default(source));
        mkdirp.sync(dest);
        const npmPackageJson = path_1.join(dest, 'package.json');
        if (!fileSystem_1.default.isFile(npmPackageJson)) {
            yield fileSystem_1.default.writeFile(npmPackageJson, JSON.stringify({
                name: 'edam-vendor-cache',
                private: true
            }, null, 2));
        }
        modulePath = path_1.join(dest, 'node_modules', source.url);
        oldPkgPath = path_1.join(modulePath, 'package.json');
        if (!cacheDir) {
            yield install();
            return modulePath;
        }
        let oldPkg;
        if (fileSystem_1.default.existsSync(oldPkgPath)) {
            oldPkg = JSON.parse(yield fileSystem_1.default.readFile(oldPkgPath, { encoding: 'utf8' }));
        }
        if (oldPkg && source.version) {
            debug('cache package.json version: %s@%s', name, oldPkg.version);
            debug('source package.json version: %s@%s', name, source.version);
            if (semver.satisfies(oldPkg.version, source.version)) {
                debug('cache satisfied!');
                log('Cached version %s of %s is matched the version range %s that you wanted.', c.cyan.bold(oldPkg.version), c.magenta.bold(source.url), c.gray.bold(source.version));
                // use cache
                return modulePath;
            }
        }
        yield install();
        return modulePath;
    });
};
