"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const nps = require("path");
const fileSystem_1 = require("../lib/fileSystem");
const EdamError_1 = require("./EdamError");
const tildify = require('tildify');
const mkdirp = require('mkdirp');
const debug = require('debug')('edam:storePrompts');
function getFilename(cacheDir) {
    return nps.join(cacheDir, 'edam-prompts.json');
}
function _get(cacheDir) {
    return __awaiter(this, void 0, void 0, function* () {
        const filename = getFilename(cacheDir);
        debug('(filename): %s', filename);
        mkdirp.sync(nps.dirname(filename));
        try {
            if (!fileSystem_1.default.existsSync(filename)) {
                yield fileSystem_1.default.writeFile(filename, '{}');
                return {};
            }
            return JSON.parse(yield fileSystem_1.default.readFile(filename, { encoding: 'utf8' }));
        }
        catch (err) {
            throw new EdamError_1.default(`Parse JSON store prompts: "${tildify(filename)}" error. please check it manually.`);
        }
    });
}
function getKeyBySource(source) {
    let key;
    switch (source.type) {
        case 'git':
            key = source.url + '?checkout=' + source.checkout;
            break;
        case 'npm':
            key = source.url + '?version=' + source.version;
            break;
        case 'file':
            key = source.url;
            break;
        default:
            key = source.url;
    }
    return key;
}
function _store(cacheDir, source, promptValues) {
    return __awaiter(this, void 0, void 0, function* () {
        const filename = getFilename(cacheDir);
        const old = yield _get(cacheDir);
        let key = getKeyBySource(source);
        old[key] = promptValues;
        yield fileSystem_1.default.writeFile(filename, JSON.stringify(old));
    });
}
function store(promptValues, prompts, { source, cacheDir } = {}) {
    return __awaiter(this, void 0, void 0, function* () {
        if (!cacheDir) {
            return;
        }
        const deniesStoreNames = prompts.filter(x => !!x.deniesStore).map(x => x.name);
        promptValues = Object.assign({}, promptValues);
        deniesStoreNames.forEach(name => {
            if (promptValues.hasOwnProperty(name)) {
                delete promptValues[name];
            }
        });
        debug('(store): promptValues %O, deniesStoreNames: %O', promptValues, deniesStoreNames);
        yield _store(cacheDir, source, promptValues);
    });
}
exports.store = store;
function get({ source, prompts = [], cacheDir } = {}) {
    return __awaiter(this, void 0, void 0, function* () {
        if (!cacheDir) {
            return;
        }
        let old = (yield _get(cacheDir))[getKeyBySource(source)];
        old = Object.assign({}, old);
        const deniesStoreNames = prompts.filter(x => !!x.deniesStore).map(x => x.name);
        deniesStoreNames.forEach(name => {
            if (old.hasOwnProperty(name)) {
                delete old[name];
            }
        });
        debug('(get): promptValues %O', old);
        return old;
    });
}
exports.get = get;
