"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * @file: index
 * @author: Cuttle Cong
 * @date: 2018/1/26
 * @description:
 */
const normalizeConfig_1 = require("./core/normalizeConfig");
const Options_1 = require("./types/Options");
const core_1 = require("./types/core");
const presetSourcePull = require("./core/pull/preset");
const EdamError_1 = require("./core/EdamError");
const coreExported = require("./core/index");
const libExported = require("./lib/index");
const constant_1 = require("./core/constant");
const nps = require("path");
const normalize_1 = require("./core/plugins/normalize");
const plugins_1 = require("./core/plugins");
const getTemplateConfig_1 = require("./lib/getTemplateConfig");
const pReduce = require("p-reduce");
const index_1 = require("./core/Compiler/index");
const promptProcessor_1 = require("./core/promptProcessor");
const FileProcessor_1 = require("./core/TreeProcessor/FileProcessor");
const DefaultLogger_1 = require("./core/DefaultLogger");
const _ = require("lodash");
const changeCase = require("change-case");
const symbolic_1 = require("./lib/symbolic");
const storePrompts_1 = require("./core/storePrompts");
const fileSystem_1 = require("./lib/fileSystem");
const lib_1 = require("./lib");
const inquirer = require('inquirer');
const tildify = require('tildify');
const debug = require('debug')('edam:core');
function throwEdamError(err, message) {
    if (err && err.id === 'EDAM_ERROR') {
        throw err;
    }
    throw new EdamError_1.default(`${message} ${err.stack}`);
}
class Edam extends core_1.AwaitEventEmitter {
    constructor(config, options = {}) {
        super();
        this.config = config;
        this.options = options;
        this.inquirer = inquirer;
        this.promptProcess = require('./core/promptProcessor/cli/index')
            .default;
        this.prompt = promptProcessor_1.default;
        this.templateConfig = {
            ignore: [],
            loaders: {},
            mappers: [],
            root: '',
            usefulHook: {
                gitInit: false,
                installDependencies: false,
                installDevDependencies: false
            }
        };
        this.compiler = new index_1.default();
        config && this.setConfig(config);
        options && this.setOption(options);
        // fill
        this.utils = Object.assign({}, Edam.utils);
        this.sourcePullMethods = Object.assign({}, Edam.sourcePullMethods);
        this.constants = Object.assign({}, Edam.constants);
        this.logger = new DefaultLogger_1.default();
        // proxy
        symbolic_1.default(this.compiler, 'logger', [this, 'logger']);
        symbolic_1.default(this.logger, 'silent', [this, ['config', 'silent']]);
        symbolic_1.default(this.compiler, 'hookCwd', [this, ['config', 'output']]);
        symbolic_1.default(this.compiler, 'root', [this, ['templateConfig', 'root']]);
    }
    normalizeConfig() {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.emit('normalizeConfig:before', this.config, this.options);
            const { track, config } = yield normalizeConfig_1.default(this.config, this.options);
            this.config = config;
            this.track = track;
            yield this.emit('normalizeConfig:after', this.config);
            return this.config;
        });
    }
    use(plugin, options = { force: false, removeExisted: true }) {
        if (!Array.isArray(plugin)) {
            plugin = [plugin, {}];
        }
        const index = this.config.plugins.findIndex(([p]) => p === plugin[0]);
        if (options.force || index < 0) {
            if (options.force && index >= 0 && options.removeExisted) {
                this.config.plugins.splice(index, 1);
            }
            this.config.plugins.push(plugin);
        }
        return this;
    }
    unuse(pluginCore) {
        this.config.plugins = this.config.plugins.filter(([p]) => p !== pluginCore);
        return this;
    }
    setConfig(config) {
        this.config = config;
        return this;
    }
    setOption(options) {
        this.options = options;
        return this;
    }
    _promptPrivate(prompts = this.templateConfig.prompts) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!prompts) {
                prompts = [];
            }
            try {
                const context = Object.assign({}, this.constants.DEFAULT_CONTEXT, { absoluteDir: this.config.output, dirName: this.config.output && nps.dirname(this.config.output), baseName: this.config.output && nps.basename(this.config.output) });
                if (this.config.storePrompts && this.config.cacheDir) {
                    let oldPromptValues = yield storePrompts_1.get({
                        source: this.config.source,
                        cacheDir: this.config.cacheDir,
                        prompts
                    });
                    oldPromptValues &&
                        Object.keys(oldPromptValues).length &&
                        prompts.forEach(prom => {
                            const { name } = prom;
                            if (typeof oldPromptValues[name] !== 'undefined') {
                                prom.default = oldPromptValues[name];
                            }
                        });
                }
                yield this.emit('prompt:before', prompts, context);
                const promptValues = yield this.prompt.call(this, prompts, {
                    yes: this.config.yes,
                    context,
                    promptProcess: this.promptProcess
                });
                this.compiler.variables.setStore(promptValues);
                if (this.config.storePrompts && this.config.cacheDir) {
                    yield storePrompts_1.store(promptValues, prompts, {
                        source: this.config.source,
                        cacheDir: this.config.cacheDir
                    });
                }
                let respectNpm5 = this.config.pull
                    ? this.config.pull.npmClient === 'npm'
                    : true;
                this.compiler.variables.merge({
                    _: Object.assign({}, _, changeCase, context, { install: (deps, opts) => lib_1.yarnInstall(deps, Object.assign({}, opts, { respectNpm5 })) })
                });
                yield this.emit('prompt:after', this.compiler.variables);
            }
            catch (err) {
                throwEdamError(err, 'Error occurs when prompting \n');
            }
        });
    }
    pull(source) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                this.config.source = source || this.config.source;
                source = this.config.source;
                if (!source) {
                    throw new EdamError_1.default('the `source` is required');
                }
                const pullMethod = this.sourcePullMethods[source.type];
                if (typeof pullMethod !== 'function') {
                    throw new EdamError_1.default(`source pull method is not found of type: ${source.type}`);
                }
                yield this.emit('pull:before', source);
                let templateConfigPath = yield pullMethod.apply(this, [
                    source,
                    this.config.cacheDir || this.constants.DEFAULT_CACHE_DIR,
                    this.config
                ]);
                templateConfigPath = this.templateConfigPath = require.resolve(templateConfigPath);
                yield this.emit('pull:after', templateConfigPath);
            }
            catch (err) {
                throwEdamError(err, 'Error occurs when pulling \n');
            }
        });
    }
    ready(source) {
        return __awaiter(this, void 0, void 0, function* () {
            this.config.source = source || this.config.source;
            yield this.normalizeConfig();
            yield this.registerPlugins();
            return this;
        });
    }
    checkConfig() {
        const normalized = this.config;
        if (!normalized.source) {
            throw new EdamError_1.default('Sorry, edam requires `source`');
        }
        if (typeof normalized.output !== 'string') {
            throw new EdamError_1.default('`config.output` requires dir path, but ' + typeof normalized.output);
        }
        normalized.output = nps.resolve(this.options.cwd, normalized.output);
        if (fileSystem_1.default.isFile(normalized.output)) {
            throw new EdamError_1.default('`config.output` requires dir path, but "' +
                tildify(normalized.output) +
                '" is a file now');
        }
        // if (!['npm', 'yarn'].includes(normalized.pull.npmClient)) {
        //   throw new EdamError(
        //     `config.pull.npmClient allows the value which is one of 'npm' | 'yarn'. but ${
        //       normalized.pull.npmClient
        //     }`
        //   )
        // }
        const unlawfulString = Options_1.edam.toUnlawfulString(normalized);
        if (unlawfulString) {
            throw new EdamError_1.default(unlawfulString);
        }
    }
    run(source) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.ready(source);
            yield this.checkConfig();
            yield this.pull();
            return yield this.process();
        });
    }
    runPlugin(plugin) {
        return __awaiter(this, void 0, void 0, function* () {
            let fn = plugin;
            let options = {};
            if (Array.isArray(plugin)) {
                fn = plugin[0];
                options = plugin[1];
            }
            return yield fn.apply(this, [options, this]);
        });
    }
    registerPlugins(plugins = this.config.plugins) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                yield pReduce((plugins || []).filter(Boolean), (_, plugin) => __awaiter(this, void 0, void 0, function* () {
                    yield this.runPlugin(plugin);
                }));
            }
            catch (err) {
                throwEdamError(err, 'Error occurs when register Plugin: \n');
            }
        });
    }
    process(templateConfigPath = this.templateConfigPath) {
        return __awaiter(this, void 0, void 0, function* () {
            // preset plugins only do something about template
            yield this.registerPlugins(plugins_1.default);
            debug('templatePath: %s', templateConfigPath);
            let templateConfig = (yield getTemplateConfig_1.default.apply(this, [require(templateConfigPath), [this, this]])) || {};
            debug('templateConfig: \n%O', templateConfig);
            // The below process would update templateConfig
            // So we requires clone
            templateConfig = _.cloneDeep(templateConfig);
            this.templateConfigPath = templateConfigPath = require.resolve(templateConfigPath);
            yield this._promptPrivate(templateConfig.prompts);
            this.templateConfig = yield normalize_1.default.apply(this, [
                templateConfig,
                templateConfigPath
            ]);
            let tree;
            try {
                yield this.emit('compiler:before');
                tree = yield this.compiler.run();
                yield this.emit('compiler:after', tree);
            }
            catch (err) {
                throwEdamError(err, 'Error occurs when compiling \n');
            }
            const fp = new FileProcessor_1.default(tree, this.config.output, this.compiler);
            fp.logger = this.logger;
            ['move', 'copy'].forEach(name => {
                const config = this.templateConfig[name];
                if (_.isObject(config) && config !== null) {
                    _.each(config, function (dest, from) {
                        fp[name](from, dest);
                    });
                }
            });
            // fp.remove(this.templateConfig.remove)
            return fp;
        });
    }
}
Edam.sourcePullMethods = Object.assign({}, presetSourcePull);
Edam.utils = Object.assign({}, coreExported, libExported);
Edam.constants = Object.assign({}, constant_1.default);
exports.Edam = Edam;
function edam(config, options) {
    return new Edam(config, options);
}
var mockPrompts_1 = require("./mockPrompts");
exports.mockPrompts = mockPrompts_1.default;
var index_2 = require("./core/Compiler/index");
exports.Compiler = index_2.default;
var FileProcessor_2 = require("./core/TreeProcessor/FileProcessor");
exports.FileProcessor = FileProcessor_2.default;
Object.assign(edam, Edam);
exports.default = edam;
