"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * @file fileSystem
 * @author Cuttle Cong
 * @date 2018/3/22
 * @description
 */
const nps = require('path');
const fs = require('fs');
const pify = require('pify');
const _ = require('lodash');
const rimraf = require('rimraf');
const match = require('./match').default;
function isFile(filename) {
    return fs.existsSync(filename) && fs.statSync(filename).isFile();
}
function isDirectory(filename) {
    return fs.existsSync(filename) && fs.statSync(filename).isDirectory();
}
const readdir = pify(fs.readdir);
function readdirDeep(path, options) {
    return __awaiter(this, void 0, void 0, function* () {
        options = options || {};
        let files = [path];
        if (isDirectory(path)) {
            files = yield readdir(path, options);
            files = files.map(f => nps.join(path, f));
            for (let i = 0; i < files.length; i++) {
                if (isDirectory(files[i])) {
                    const newFiles = yield readdirDeep(files[i], Object.assign({}, options, { filter: null }));
                    files.splice(i, 1);
                    files = files.concat(newFiles);
                    i--;
                }
            }
            files = _.uniq(files);
        }
        return options.filter
            ? match(files, options.filter)
            : files;
    });
}
function cleanDir(path) {
    return __awaiter(this, void 0, void 0, function* () {
        return pify(rimraf)(path);
    });
}
exports.default = Object.assign({}, fs, { isFile,
    isDirectory, readFile: pify(fs.readFile), writeFile: pify(fs.writeFile), readdir,
    readdirDeep,
    cleanDir });
