export declare type AsyncOrSync<T> = Promise<T> | T;
export declare type PromptType = 'checkbox' | 'radio' | 'input' | 'suggest';
export interface Prompt {
    message: string;
    default?: any;
    yes?: boolean;
    when?: (vars: Variables) => boolean;
    name: string;
    type: PromptType;
    choices?: Array<any>;
    transform?: Function;
    deniesStore?: boolean;
}
export declare type Hook = string | Function;
export declare type Glob = string;
export declare type VarCenter = {
    once: Function;
    always: Function;
};
export declare type GetVariable = (vc: VarCenter) => AsyncOrSync<any>;
export declare type Variable = GetVariable | any;
export declare type Variables = {
    [name: string]: Variable;
};
export declare type StrictLoader = Function & {
    raw?: boolean;
};
export declare type StrictLoaderWithOption = [StrictLoader, any];
export declare type Loader = Array<StrictLoader | string | StrictLoaderWithOption> | string | StrictLoader | StrictLoaderWithOption;
export declare type Matcher = Glob | RegExp | Function;
export declare type Mapper = {
    test: Matcher;
    loader: Loader;
};
export declare type Dynamic<T> = (answer: object) => T | Promise<T>;
export default interface TemplateConfig {
    prompts?: Array<Prompt>;
    /**
     * answers => ({ hooks, ignore, ...(exclude prompts) })
     */
    process?: Function;
    hooks?: {
        [hookName: string]: Array<Hook> | Hook;
    } | Dynamic<object>;
    ignore?: string[] | Dynamic<string[]>;
    variables?: Variables;
    root?: string | Dynamic<string>;
    loaders?: {
        [loaderId: string]: Array<StrictLoader>;
    };
    mappers?: Array<Mapper>;
    move?: Dynamic<object> | object;
    copy?: Dynamic<object> | object;
    usefulHook: {
        gitInit?: boolean;
        installDependencies?: boolean;
        installDevDependencies?: boolean;
    };
}
